/* valaccodecastexpression.vala
 *
 * Copyright (C) 2006-2008  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>


#define VALA_TYPE_CCODE_NODE (vala_ccode_node_get_type ())
#define VALA_CCODE_NODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_NODE, ValaCCodeNode))
#define VALA_CCODE_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_NODE, ValaCCodeNodeClass))
#define VALA_IS_CCODE_NODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_NODE))
#define VALA_IS_CCODE_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_NODE))
#define VALA_CCODE_NODE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_NODE, ValaCCodeNodeClass))

typedef struct _ValaCCodeNode ValaCCodeNode;
typedef struct _ValaCCodeNodeClass ValaCCodeNodeClass;
typedef struct _ValaCCodeNodePrivate ValaCCodeNodePrivate;

#define VALA_TYPE_CCODE_WRITER (vala_ccode_writer_get_type ())
#define VALA_CCODE_WRITER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_WRITER, ValaCCodeWriter))
#define VALA_CCODE_WRITER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_WRITER, ValaCCodeWriterClass))
#define VALA_IS_CCODE_WRITER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_WRITER))
#define VALA_IS_CCODE_WRITER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_WRITER))
#define VALA_CCODE_WRITER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_WRITER, ValaCCodeWriterClass))

typedef struct _ValaCCodeWriter ValaCCodeWriter;
typedef struct _ValaCCodeWriterClass ValaCCodeWriterClass;

#define VALA_TYPE_CCODE_EXPRESSION (vala_ccode_expression_get_type ())
#define VALA_CCODE_EXPRESSION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_EXPRESSION, ValaCCodeExpression))
#define VALA_CCODE_EXPRESSION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_EXPRESSION, ValaCCodeExpressionClass))
#define VALA_IS_CCODE_EXPRESSION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_EXPRESSION))
#define VALA_IS_CCODE_EXPRESSION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_EXPRESSION))
#define VALA_CCODE_EXPRESSION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_EXPRESSION, ValaCCodeExpressionClass))

typedef struct _ValaCCodeExpression ValaCCodeExpression;
typedef struct _ValaCCodeExpressionClass ValaCCodeExpressionClass;
typedef struct _ValaCCodeExpressionPrivate ValaCCodeExpressionPrivate;

#define VALA_TYPE_CCODE_CAST_EXPRESSION (vala_ccode_cast_expression_get_type ())
#define VALA_CCODE_CAST_EXPRESSION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_CAST_EXPRESSION, ValaCCodeCastExpression))
#define VALA_CCODE_CAST_EXPRESSION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_CAST_EXPRESSION, ValaCCodeCastExpressionClass))
#define VALA_IS_CCODE_CAST_EXPRESSION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_CAST_EXPRESSION))
#define VALA_IS_CCODE_CAST_EXPRESSION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_CAST_EXPRESSION))
#define VALA_CCODE_CAST_EXPRESSION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_CAST_EXPRESSION, ValaCCodeCastExpressionClass))

typedef struct _ValaCCodeCastExpression ValaCCodeCastExpression;
typedef struct _ValaCCodeCastExpressionClass ValaCCodeCastExpressionClass;
typedef struct _ValaCCodeCastExpressionPrivate ValaCCodeCastExpressionPrivate;
#define _vala_ccode_node_unref0(var) ((var == NULL) ? NULL : (var = (vala_ccode_node_unref (var), NULL)))
#define _g_free0(var) (var = (g_free (var), NULL))

struct _ValaCCodeNode {
	GTypeInstance parent_instance;
	volatile int ref_count;
	ValaCCodeNodePrivate * priv;
};

struct _ValaCCodeNodeClass {
	GTypeClass parent_class;
	void (*finalize) (ValaCCodeNode *self);
	void (*write) (ValaCCodeNode* self, ValaCCodeWriter* writer);
	void (*write_declaration) (ValaCCodeNode* self, ValaCCodeWriter* writer);
	void (*write_combined) (ValaCCodeNode* self, ValaCCodeWriter* writer);
};

struct _ValaCCodeExpression {
	ValaCCodeNode parent_instance;
	ValaCCodeExpressionPrivate * priv;
};

struct _ValaCCodeExpressionClass {
	ValaCCodeNodeClass parent_class;
	void (*write_inner) (ValaCCodeExpression* self, ValaCCodeWriter* writer);
};

struct _ValaCCodeCastExpression {
	ValaCCodeExpression parent_instance;
	ValaCCodeCastExpressionPrivate * priv;
};

struct _ValaCCodeCastExpressionClass {
	ValaCCodeExpressionClass parent_class;
};

struct _ValaCCodeCastExpressionPrivate {
	ValaCCodeExpression* _inner;
	char* _type_name;
};


static gpointer vala_ccode_cast_expression_parent_class = NULL;

gpointer vala_ccode_node_ref (gpointer instance);
void vala_ccode_node_unref (gpointer instance);
GParamSpec* vala_param_spec_ccode_node (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void vala_value_set_ccode_node (GValue* value, gpointer v_object);
gpointer vala_value_get_ccode_node (const GValue* value);
GType vala_ccode_node_get_type (void);
gpointer vala_ccode_writer_ref (gpointer instance);
void vala_ccode_writer_unref (gpointer instance);
GParamSpec* vala_param_spec_ccode_writer (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void vala_value_set_ccode_writer (GValue* value, gpointer v_object);
gpointer vala_value_get_ccode_writer (const GValue* value);
GType vala_ccode_writer_get_type (void);
GType vala_ccode_expression_get_type (void);
GType vala_ccode_cast_expression_get_type (void);
#define VALA_CCODE_CAST_EXPRESSION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_CCODE_CAST_EXPRESSION, ValaCCodeCastExpressionPrivate))
enum  {
	VALA_CCODE_CAST_EXPRESSION_DUMMY_PROPERTY
};
ValaCCodeExpression* vala_ccode_expression_construct (GType object_type);
void vala_ccode_cast_expression_set_inner (ValaCCodeCastExpression* self, ValaCCodeExpression* value);
void vala_ccode_cast_expression_set_type_name (ValaCCodeCastExpression* self, const char* value);
ValaCCodeCastExpression* vala_ccode_cast_expression_new (ValaCCodeExpression* expr, const char* type);
ValaCCodeCastExpression* vala_ccode_cast_expression_construct (GType object_type, ValaCCodeExpression* expr, const char* type);
void vala_ccode_writer_write_string (ValaCCodeWriter* self, const char* s);
const char* vala_ccode_cast_expression_get_type_name (ValaCCodeCastExpression* self);
ValaCCodeExpression* vala_ccode_cast_expression_get_inner (ValaCCodeCastExpression* self);
void vala_ccode_expression_write_inner (ValaCCodeExpression* self, ValaCCodeWriter* writer);
static void vala_ccode_cast_expression_real_write (ValaCCodeNode* base, ValaCCodeWriter* writer);
void vala_ccode_node_write (ValaCCodeNode* self, ValaCCodeWriter* writer);
static void vala_ccode_cast_expression_real_write_inner (ValaCCodeExpression* base, ValaCCodeWriter* writer);
static void vala_ccode_cast_expression_finalize (ValaCCodeNode* obj);



ValaCCodeCastExpression* vala_ccode_cast_expression_construct (GType object_type, ValaCCodeExpression* expr, const char* type) {
	ValaCCodeCastExpression* self;
	g_return_val_if_fail (expr != NULL, NULL);
	g_return_val_if_fail (type != NULL, NULL);
	self = (ValaCCodeCastExpression*) vala_ccode_expression_construct (object_type);
	vala_ccode_cast_expression_set_inner (self, expr);
	vala_ccode_cast_expression_set_type_name (self, type);
	return self;
}


ValaCCodeCastExpression* vala_ccode_cast_expression_new (ValaCCodeExpression* expr, const char* type) {
	return vala_ccode_cast_expression_construct (VALA_TYPE_CCODE_CAST_EXPRESSION, expr, type);
}


static void vala_ccode_cast_expression_real_write (ValaCCodeNode* base, ValaCCodeWriter* writer) {
	ValaCCodeCastExpression * self;
	self = (ValaCCodeCastExpression*) base;
	g_return_if_fail (writer != NULL);
	vala_ccode_writer_write_string (writer, "(");
	vala_ccode_writer_write_string (writer, self->priv->_type_name);
	vala_ccode_writer_write_string (writer, ") ");
	vala_ccode_expression_write_inner (self->priv->_inner, writer);
}


static void vala_ccode_cast_expression_real_write_inner (ValaCCodeExpression* base, ValaCCodeWriter* writer) {
	ValaCCodeCastExpression * self;
	self = (ValaCCodeCastExpression*) base;
	g_return_if_fail (writer != NULL);
	vala_ccode_writer_write_string (writer, "(");
	vala_ccode_node_write ((ValaCCodeNode*) self, writer);
	vala_ccode_writer_write_string (writer, ")");
}


ValaCCodeExpression* vala_ccode_cast_expression_get_inner (ValaCCodeCastExpression* self) {
	ValaCCodeExpression* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_inner;
	return result;
}


static gpointer _vala_ccode_node_ref0 (gpointer self) {
	return self ? vala_ccode_node_ref (self) : NULL;
}


void vala_ccode_cast_expression_set_inner (ValaCCodeCastExpression* self, ValaCCodeExpression* value) {
	ValaCCodeExpression* _tmp0_;
	g_return_if_fail (self != NULL);
	self->priv->_inner = (_tmp0_ = _vala_ccode_node_ref0 (value), _vala_ccode_node_unref0 (self->priv->_inner), _tmp0_);
}


const char* vala_ccode_cast_expression_get_type_name (ValaCCodeCastExpression* self) {
	const char* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_type_name;
	return result;
}


void vala_ccode_cast_expression_set_type_name (ValaCCodeCastExpression* self, const char* value) {
	char* _tmp0_;
	g_return_if_fail (self != NULL);
	self->priv->_type_name = (_tmp0_ = g_strdup (value), _g_free0 (self->priv->_type_name), _tmp0_);
}


static void vala_ccode_cast_expression_class_init (ValaCCodeCastExpressionClass * klass) {
	vala_ccode_cast_expression_parent_class = g_type_class_peek_parent (klass);
	VALA_CCODE_NODE_CLASS (klass)->finalize = vala_ccode_cast_expression_finalize;
	g_type_class_add_private (klass, sizeof (ValaCCodeCastExpressionPrivate));
	VALA_CCODE_NODE_CLASS (klass)->write = vala_ccode_cast_expression_real_write;
	VALA_CCODE_EXPRESSION_CLASS (klass)->write_inner = vala_ccode_cast_expression_real_write_inner;
}


static void vala_ccode_cast_expression_instance_init (ValaCCodeCastExpression * self) {
	self->priv = VALA_CCODE_CAST_EXPRESSION_GET_PRIVATE (self);
}


static void vala_ccode_cast_expression_finalize (ValaCCodeNode* obj) {
	ValaCCodeCastExpression * self;
	self = VALA_CCODE_CAST_EXPRESSION (obj);
	_vala_ccode_node_unref0 (self->priv->_inner);
	_g_free0 (self->priv->_type_name);
	VALA_CCODE_NODE_CLASS (vala_ccode_cast_expression_parent_class)->finalize (obj);
}


GType vala_ccode_cast_expression_get_type (void) {
	static GType vala_ccode_cast_expression_type_id = 0;
	if (vala_ccode_cast_expression_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaCCodeCastExpressionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_ccode_cast_expression_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaCCodeCastExpression), 0, (GInstanceInitFunc) vala_ccode_cast_expression_instance_init, NULL };
		vala_ccode_cast_expression_type_id = g_type_register_static (VALA_TYPE_CCODE_EXPRESSION, "ValaCCodeCastExpression", &g_define_type_info, 0);
	}
	return vala_ccode_cast_expression_type_id;
}




